/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the Clippings extension for Mozilla Firefox.
 *
 * The Initial Developer of the Original Code is 
 * Alex Eng <ateng@users.sourceforge.net>.
 * Portions created by the Initial Developer are Copyright (C) 2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * ***** END LICENSE BLOCK ***** */

//
// Miscellaneous declarations for both the Clippings Manager window and the 
// host app's main window
//

const CLIPPINGS_DEBUG = true;
const CLIPDAT_FILE_NAME = "clipdat.rdf";

var gClippings = {
  dataSrc: null,  // RDF data source
  RDFSvc:  Components.classes["@mozilla.org/rdf/rdf-service;1"].getService(Components.interfaces.nsIRDFService),
  RDFContainerUtils: Components.classes["@mozilla.org/rdf/container-utils;1"].getService(Components.interfaces.nsIRDFContainerUtils),
  RDFContainer: null,

  newFromClipboard: function (aShowDialog) 
  {
    var clipText = this.getTextFromClipboard();
    var rv = this.newClippingFromText(clipText, aShowDialog);
    return rv;
  },

  newClippingFromText: function (aText, aShowDialog)
  {
    var rv, clipName;
    var maxNameLen = this.getPref("clippings.entries.max_name_length") || 48;
    var clipText = aText;
    if (clipText) {
      if (clipText.length > maxNameLen) {
	clipName = clipText.substr(0, maxNameLen) + "...";
      } 
      else {
	clipName = clipText;
      }

      if (aShowDialog) {
	var args = {name: clipName, text: clipText, userCancel: null};
	window.openDialog("chrome://clippings/content/new.xul", "newclp_dlg", 
			  "centerscreen,modal", args);
	if (args.userCancel) {
	  return -1;
	}
	clipName = args.name;
	clipText = args.text;
      }

      rv = this.createNewClipping(clipName, clipText);
    } 
    return rv;
  },

  getTextFromClipboard: function () 
  {
    var rv;
    var clipbd = Components.classes["@mozilla.org/widget/clipboard;1"].createInstance(Components.interfaces.nsIClipboard);
    var trans = Components.classes["@mozilla.org/widget/transferable;1"].createInstance(Components.interfaces.nsITransferable);
    trans.addDataFlavor("text/unicode");
    clipbd.getData(trans, clipbd.kGlobalClipboard);

    var str = {};
    var strLen = {};
    try {
      trans.getTransferData("text/unicode", str, strLen);
    } 
    catch (e) {
      // Failed because there's nothing on the clipboard or the clipboard
      // contents isn't textual data
      return rv;
    }

    if (str) {
      str = str.value.QueryInterface(Components.interfaces.nsISupportsString);
    }
    if (str) {
      rv = str.data.substring(0, strLen.value / 2);
    }

    return rv;
  },

  createNewClipping: function (aName, aText) 
  {
    var newNode = this.RDFSvc.GetAnonymousResource();
    this._createNewClippingHelper(newNode, aName, aText);
    return newNode.Value;  // URI of new RDF node
  },

  createNewClippingWithURI: function (aURI, aName, aText)
  {
    var newNode = this.RDFSvc.GetResource(aURI);
    this._createNewClippingHelper(newNode, aName, aText);
    return aURI;
  },
  
  _createNewClippingHelper: function (aNode, aName, aText)
  {
    var predName = this.RDFSvc.GetResource("http://clippings.mozdev.org/ns/rdf#name");
    var predText = this.RDFSvc.GetResource("http://clippings.mozdev.org/ns/rdf#text");
    var targName = this.RDFSvc.GetLiteral(aName);
    var targText = this.RDFSvc.GetLiteral(aText);

    this.dataSrc.Assert(aNode, predName, targName, true);
    this.dataSrc.Assert(aNode, predText, targText, true);
    this.RDFContainer.AppendElement(aNode);
  },

  flushDataSrc: function ()  // Throws exception if datasource flush failed.
  {
    try {
      gClippings.dataSrc.QueryInterface(Components.interfaces.nsIRDFRemoteDataSource).Flush();
    }
    catch (e) {
      if (CLIPPINGS_DEBUG) {
	alert(e);
      }
      throw e;
      return false;
    }
    gClippings.log("*** Clippings datasource saved to disk ***");
    return true;
  },

  copyTextToClipboard: function (aText) 
  {
    var clipbd = Components.classes["@mozilla.org/widget/clipboardhelper;1"].getService(Components.interfaces.nsIClipboardHelper);
    clipbd.copyString(aText);
  },

  getPref: function (aPrefName)
  {
    var pref = Components.classes["@mozilla.org/preferences-service;1"]
                         .getService(Components.interfaces.nsIPrefBranch);
    try {
      var prefType = pref.getPrefType(aPrefName);
    } catch (e) {
      this.log(e);
    }
    var rv;
    switch (prefType) {
      case pref.PREF_STRING:
      rv = pref.getCharPref(aPrefName);
      break;

      case pref.PREF_INT:
      rv = pref.getIntPref(aPrefName);
      break;

      case pref.PREF_BOOL:
      rv = pref.getBoolPref(aPrefName);
      break;

      default:
      break;
    }
    return rv;
  },
  
  getCurrentProfileURL: function ()  
  // Throws an exception if profile directory retrieval failed.
  {
    var dirSvc = Components.classes["@mozilla.org/file/directory_service;1"].getService(Components.interfaces.nsIDirectoryService);
    var dirProp = dirSvc.QueryInterface(Components.interfaces.nsIProperties);
    var profileDir = dirProp.get("ProfD", Components.interfaces.nsIFile);
    if (! profileDir) {
      throw "Failed to retrieve current profile directory path";
    }

    var fph = Components.classes["@mozilla.org/network/protocol;1?name=file"].createInstance(Components.interfaces.nsIFileProtocolHandler);
    return fph.getURLSpecFromFile(profileDir);
  },

  beep: function () 
  {
    var sound = Components.classes["@mozilla.org/sound;1"].createInstance(Components.interfaces.nsISound);
    sound.beep();
  },

  log: function (aMessage) 
  {
    if (CLIPPINGS_DEBUG) {
      var consoleSvc = Components.classes["@mozilla.org/consoleservice;1"].getService(Components.interfaces.nsIConsoleService);
      consoleSvc.logStringMessage(aMessage);
    }
  }
};
